/*
**==============================================================================
**
** Open Management Infrastructure (OMI)
**
** Copyright (c) Microsoft Corporation
** 
** Licensed under the Apache License, Version 2.0 (the "License"); you may not 
** use this file except in compliance with the License. You may obtain a copy 
** of the License at 
**
**     http://www.apache.org/licenses/LICENSE-2.0 
**
** THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
** KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED 
** WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABLITY OR NON-INFRINGEMENT. 
**
** See the Apache 2 License for the specific language governing permissions 
** and limitations under the License.
**
**==============================================================================
*/

#ifndef _omi_strarr_h
#define _omi_strarr_h

#include <common.h>
#include <wchar.h>

#if defined(CONFIG_ENABLE_WCHAR)
# define ZArr WcsArr
# define ZArrCat WcsArrCat
# define ZArrLen WcsArrLen
# define ZArrFree WcsArrFree
#else
# define ZArr StrArr
# define ZArrCat StrArrCat
# define ZArrLen StrArrLen
# define ZArrFree StrArrFree
#endif

BEGIN_EXTERNC

/* Allocate an empty null-terminated string array */
char** StrArr();

/* Append a string to the end of the string array */
void StrArrCat(char*** self, const char* str);

/* Return length of this array */
size_t StrArrLen(char** self);

/* Free the string array */
void StrArrFree(char** self);

/* Allocate an empty null-terminated string array */
wchar_t** WcsArr();

/* Append a string to the end of the string array */
void WcsArrCat(wchar_t*** self, const wchar_t* str);

/* Return length of this array */
size_t WcsArrLen(wchar_t** self);

/* Free the string array */
void WcsArrFree(wchar_t** self);

END_EXTERNC

#endif /* _omi_strarr_h */
